﻿using Syncfusion.Grouping;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace CustomGrouping
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            //Set DataSource for the GridControl
            this.gridGroupingControl1.DataSource = GetADataTable();
            //Set Grid Properties
            this.gridGroupingControl1.ShowGroupDropArea = true;
            this.gridGroupingControl1.TopLevelGroupOptions.ShowAddNewRecordBeforeDetails = false;

            //color "Col2" so it is easier to see
            this.gridGroupingControl1.TableDescriptor.Columns["Col2"].Appearance.AnyRecordFieldCell.BackColor = Color.LightGoldenrodYellow;

            //group "Col2" using a custom categorizer and Comparer
            Syncfusion.Grouping.SortColumnDescriptor cd = new Syncfusion.Grouping.SortColumnDescriptor("Col2");
            cd.Categorizer = new CustomCategorizer();
            cd.Comparer = new CustomComparer();
            this.gridGroupingControl1.TableDescriptor.GroupedColumns.Add(cd);

            //subcribe to event to display custom group caption
            this.gridGroupingControl1.QueryCellStyleInfo += new Syncfusion.Windows.Forms.Grid.Grouping.GridTableCellStyleInfoEventHandler(gridGroupingControl1_QueryCellStyleInfo);
        }

        #region [DataTable]
        private DataTable GetADataTable()
        {

            DataTable dt = new DataTable("MyTable");

            int nCols = 4;
            int nRows = 50;

            for (int i = 0; i < nCols; i++)
                dt.Columns.Add(new DataColumn(string.Format("Col{0}", i)));

            Random r = new Random();
            for (int i = 0; i < nRows; ++i)
            {
                DataRow dr = dt.NewRow();
                for (int j = 0; j < nCols; j++)
                {
                    if (j == 2)
                    {
                        if (i < 3)
                        {
                            dr[j] = "Group0";
                        }
                        else if (i < 6)
                        {
                            dr[j] = "Group1";
                        }
                        else
                        {
                            dr[j] = "Group" + i;
                        }
                    }
                    else
                    dr[j] = r.Next(100).ToString();
                }
                dt.Rows.Add(dr);
            }

            return dt;
        }
        #endregion

        #region [QueryCellStyleInfo Event Handler]
        //handle QueryCellStyleInfo to display custom text to match the custom categorizer
        private void gridGroupingControl1_QueryCellStyleInfo(object sender, Syncfusion.Windows.Forms.Grid.Grouping.GridTableCellStyleInfoEventArgs e)
        {
            if (e.TableCellIdentity.GroupedColumn != null && e.TableCellIdentity.DisplayElement.ParentGroup != null
                && e.TableCellIdentity.DisplayElement.ParentGroup.Category is int)
            {
                if (e.TableCellIdentity.DisplayElement is CaptionRow
                    && e.TableCellIdentity.GroupedColumn.Name == "Col2")
                {
                    int cat = (int)e.TableCellIdentity.DisplayElement.ParentGroup.Category;
                    string ret = "";
                    switch (cat)
                    {
                        case 1:
                            ret = "Group0";
                            break;
                        case 2:
                            ret = "Group1";
                            break;
                        case 3:
                            ret = "Groups";
                            break;                        
                    }
                    e.Style.CellValue = String.Format("{0}: {1} Items.", ret, e.TableCellIdentity.DisplayElement.ParentGroup.GetChildCount());
                }
            }
        }
        #endregion

    }
    #region Custom Comparer class
    //make sure the string integers are sorted as integers instead of strings
    public class CustomComparer : IComparer
    {
        #region IComparer Members

        public int Compare(object x, object y)
        {

            if (x == null && y == null)
                return 0;
            else
                return ((IComparable)x.ToString()).CompareTo(y.ToString());
        }

        #endregion
    }

    #endregion

    #region Custom Categorizer class
    //defines custom categorizer
    public class CustomCategorizer : Syncfusion.Grouping.IGroupByColumnCategorizer
    {

        //defines a group and returns a group category object (here returns 1 through 5)
        public static int GetCategory(string i)
        {
            int ret = 0;

            if (i == "Group0")
            {
                ret = 1;
            }
            else if (i == "Group1")
            {
                ret = 2;
            }
            else
                ret = 3;

            return ret;
        }

        #region IGroupByColumnCategorizer Members

        public object GetGroupByCategoryKey(SortColumnDescriptor column, bool isForeignKey, Record record)
        {
            return GetCategory(record.GetValue(column).ToString());
        }

        public int CompareCategoryKey(SortColumnDescriptor column, bool isForeignKey, object category, Record record)
        {
            return GetCategory(record.GetValue(column).ToString()) - (int)category;
        }

        #endregion
    }
    #endregion
}